import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Popover, Switch, Col, Modal, Button } from 'antd'
import { PlusOutlined, PlusSquareOutlined, EditOutlined, ArrowLeftOutlined, ArrowRightOutlined, SwapOutlined, ToolOutlined, DeleteOutlined, FontColorsOutlined } from '@ant-design/icons'

import asyncComponent from '@/utils/asyncComponent'
import asyncIconComponent from '@/utils/asyncIconComponent'
import { resetStyle } from '@/utils/utils-custom.js'
import getSettingForm from './options'
import Utils from '@/utils/utils.js'
import MKEmitter from '@/utils/events.js'
import './index.scss'

const { confirm } = Modal

const NormalForm = asyncIconComponent(() => import('@/components/normalform'))
const CardCellComponent = asyncComponent(() => import('@/menu/components/card/cardcellcomponent'))
const CopyComponent = asyncIconComponent(() => import('@/menu/components/share/copycomponent'))
const PasteController = asyncIconComponent(() => import('@/components/paste'))

class CardBoxComponent extends Component {
  static propTpyes = {
    cards: PropTypes.object,         // 卡片行配置信息
    card: PropTypes.object,          // 卡片配置信息
    move: PropTypes.func,            // 卡片移动
    deleteElement: PropTypes.func,   // 卡片删除
    updateElement: PropTypes.func    // 菜单配置更新
  }

  state = {
    card: null,            // 卡片信息，包括正反面
    formlist: null,        // 设置表单信息
    elements: null,        // 编辑组
    side: 'front',
    timestamp: '',
    appType: sessionStorage.getItem('appType'),
    visible: false
  }

  /**
   * @description 搜索条件初始化
   */
  UNSAFE_componentWillMount () {
    const { card } = this.props

    this.setState({
      timestamp: new Date().getTime() + '',
      card: fromJS(card).toJS(),
      elements: fromJS(card.elements).toJS(),
    })
  }

  shouldComponentUpdate (nextProps, nextState) {
    const { cards } = this.props
    
    return !is(fromJS(cards), fromJS(nextProps.cards)) || !is(fromJS(this.state), fromJS(nextState))
  }

  /**
   * @description 组件销毁，清除state更新，清除快捷键设置
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
  }

  getStyle = (style) => {
    const { card, side } = this.state

    let _card = fromJS(card).toJS()
    if (side === 'back') {
      _card.backStyle = style
    } else {
      _card.style = style
    }

    this.setState({
      card: _card
    })

    this.props.updateElement(_card)
  }

  updateCard = (elements, btn) => {
    const { card, side } = this.state

    let _card = {}

    if (side === 'back') {
      _card = {...card, backElements: elements}
    } else {
      _card = {...card, elements: elements}
    }

    this.setState({
      card: _card
    })

    this.props.updateElement(_card, btn)
  }

  changeSide = () => {
    const { card } = this.props
    const { side } = this.state

    let _side = ''
    let _elements = null
    if (side === 'front') {
      _side = 'back'
    } else {
      _side = 'front'
    }

    if (_side === 'front') {
      _elements = fromJS(card.elements).toJS()
    } else {
      _elements = fromJS(card.backElements).toJS()
    }

    this.setState({side: _side, elements: _elements, timestamp: new Date().getTime() + ''})
  }
  
  addElement = () => {
    const { cards } = this.props
    const { card } = this.state

    let newcard = {}
    newcard.uuid = Utils.getuuid()
    newcard.focus = true
    
    newcard.eleType = 'text'
    newcard.datatype = 'dynamic'
    newcard.height = 1

    if (card.$cardType === 'extendCard' && card.setting.cardRole === 'header') {
      newcard.datatype = 'static'
    } else if (cards.subtype === 'propcard' && cards.wrap.datatype === 'static') {
      newcard.datatype = 'static'
    }

    // 注册事件-添加元素
    MKEmitter.emit('cardAddElement', card.uuid, newcard)
  }

  addButton = () => {
    const { card } = this.state

    let newcard = {eleType: 'button', label: 'button', verify: null, show: 'link', sqlType: '', Ot: 'requiredSgl', OpenType: 'prompt', icon: '', class: 'primary', intertype: 'system', execSuccess: 'grid', execError: 'never', popClose: 'never'}
    newcard.uuid = Utils.getuuid()
    newcard.focus = true

    // 注册事件-添加元素
    MKEmitter.emit('cardAddElement', card.uuid, newcard)
  }

  changeStyle = () => {
    const { card, side } = this.state

    let _style = null
    let options = ['height', 'background', 'border', 'padding', 'margin', 'shadow', 'clear', 'overflow']
    if (side === 'front') {
      _style = card.style ? fromJS(card.style).toJS() : {}
    } else if (side === 'back') {
      _style = card.backStyle ? fromJS(card.backStyle).toJS() : {}
      options = ['background', 'padding']
    }

    MKEmitter.emit('changeStyle', options, _style, this.getStyle)
  }

  getSettingForms = () => {
    const { cards } = this.props
    const { card, appType } = this.state

    let buttons = []
    card.elements && card.elements.forEach(item => {
      if (item.eleType === 'button') {
        buttons.push({
          value: item.uuid,
          label: item.label
        })
      }
    })
    if (appType !== 'mob' && card.backElements) {
      card.backElements.forEach(item => {
        if (item.eleType === 'button') {
          buttons.push({
            value: item.uuid,
            label: item.label
          })
        }
      })
    }
    return getSettingForm(card.setting, cards.subtype, buttons, card.$cardType, cards.columns, card.menus)
  }

  updateSetting = (res) => {
    const { card, side, appType } = this.state

    if (card.$cardType === 'extendCard' && res.cardRole === 'header') {
      let _card = {...card, setting: res}

      let originLength = _card.elements.length
      _card.elements = _card.elements.filter(item => item.eleType === 'text')
      _card.backElements = []

      delete _card.menus

      if (_card.elements.length < originLength) {
        const that = this

        confirm({
          title: '表格头仅支持文本，确定要切换卡片角色吗?',
          content: '',
          okText: '确定',
          cancelText: '取消',
          onOk() {
            that.setState({ card: _card, side: 'front', timestamp: new Date().getTime() + '', elements: fromJS(_card.elements).toJS() })
            that.props.updateElement(_card)
          },
          onCancel() {}
        })
      } else {
        this.setState({ card: _card, side: 'front' })
        this.props.updateElement(_card)
      }

      return
    }

    if (appType === '' && res.menu) {
      let list = null
      try {
        list = JSON.parse(sessionStorage.getItem('thdMenuList')) || []
      } catch (e) {
        list = []
      }

      let id = res.menu[res.menu.length - 1]

      list.forEach(item => {
        if (item.MenuID === id) {
          res.MenuID = id
          res.MenuName = item.MenuName
          res.MenuNo = item.MenuNo
          res.tabType = item.type
        }
      })
    }

    let _card = {...card, setting: res}

    if (res.menus) {
      _card.menus = res.menus

      delete res.menus
    } else {
      delete _card.menus
    }

    this.setState({
      card: _card
    })

    if (side === 'back' && res.type === 'simple') {
      this.setState({
        side: 'front',
        timestamp: new Date().getTime() + '',
        elements: fromJS(_card.elements).toJS()
      })
    }

    this.props.updateElement(_card)
  }

  updateFields = (elements, resolve, type) => {
    const { card, side } = this.state

    let items = elements

    if (type === 'form') {
      items = []

      elements.forEach(item => {
        let cell = {
          uuid: Utils.getuuid(),
          eleType: 'text',
          height: 1,
          innerHeight: 21,
          prefix: item.label ? item.label + ': ' : '',
          datatype: item.field ? 'dynamic' : 'static',
          field: item.field || '',
          width: item.span || 12,
          value: '',
          style: {marginBottom: '15px'}
        }

        if (item.type === 'number') {
          cell.eleType = 'number'
          cell.decimal = typeof(item.decimal) === 'number' ? item.decimal : ''
        }

        items.push(cell)
      })
    }

    if (items.length === 0) {
      resolve({status: false, message: '元素不可为空！'})
      return
    }

    let _card = { ...card }
    let _card2 = { ...card }
    let pass = false

    if (side === 'back') {
      if (!_card.backElements || _card.backElements.length === 0) {
        _card.backElements = items
        pass = true
      } else {
        _card.backElements = items
        _card2.backElements = [...card.backElements, ...items]
      }
    } else {
      if (!_card.elements || _card.elements.length === 0) {
        _card.elements = items
        pass = true
      } else {
        _card.elements = items
        _card2.elements = [...card.elements, ...items]
      }
    }

    if (pass) {
      this.setState({
        card: _card,
        timestamp: new Date().getTime() + '',
        elements: side === 'back' ? fromJS(_card.backElements).toJS() : fromJS(_card.elements).toJS()
      })

      this.props.updateElement(_card)
    } else {
      const that = this
      const _modal = confirm({
        title: '卡片中已存在元素，请选择要进行的操作。',
        className: 'mk-confirm-modal',
        content: <div className="footer">
          <Button key="cancel" onClick={() => _modal.destroy()}>取消</Button>
          <Button key="replace" className="mk-border-purple" onClick={() => {
            that.setState({
              card: _card,
              timestamp: new Date().getTime() + '',
              elements: side === 'back' ? fromJS(_card.backElements).toJS() : fromJS(_card.elements).toJS()
            })
      
            that.props.updateElement(_card)
            _modal.destroy()
          }}>替换</Button>
          <Button key="confirm" className="mk-border-green" onClick={() => {
            that.setState({
              card: _card2,
              timestamp: new Date().getTime() + '',
              elements: side === 'back' ? fromJS(_card2.backElements).toJS() : fromJS(_card2.elements).toJS()
            })
      
            that.props.updateElement(_card2)
            _modal.destroy()
          }}>添加</Button>
        </div>
      })
    }

    resolve({status: true})
  }

  paste = (element, resolve) => {
    const { card } = this.state

    if (['simpleform', 'forms', 'formgroup', 'cardcell'].includes(element.copyType)) {
      if (element.copyType === 'simpleform') {
        this.updateFields(element.subcards[0].fields, resolve, 'form')
      } else if (element.copyType === 'forms') {
        this.updateFields(element.fields, resolve, 'form')
      } else if (element.copyType === 'formgroup') {
        this.updateFields(element.fields, resolve, 'form')
      } else {
        this.updateFields(element.elements, resolve)
      }
      return
    }

    let _uuid = Utils.getuuid()
    
    if (card.$cardType === 'extendCard' && card.setting.cardRole === 'header' && element.eleType !== 'text') {
      resolve({status: false, message: '表格头仅支持文本元素！'})
      return
    }

    if (element.copyType === 'action') {
      element.eleType = 'button'
      element.width = element.width || 12
    }

    element.uuid = _uuid
    element.focus = true

    resolve({status: true})

    // 注册事件-添加元素
    MKEmitter.emit('cardAddElement', card.uuid, element)
  }

  doubleClickCard = () => {
    const { cards } = this.props
    const { card, appType } = this.state

    if (card.setting.click === 'menu' && card.setting.menu) {
      if (['IM'].includes(card.setting.menu)) {
        if (!sessionStorage.getItem('instantMessage')) return
        
        let param = {
          MenuID: sessionStorage.getItem('instantMessage'),
          copyMenuId: '',
          type: 'view'
        }
    
        param = window.btoa(window.encodeURIComponent(JSON.stringify(param)))
    
        MKEmitter.emit('changeEditMenu', {routerUrl: '/imdesign/' + param})
      } else {
        MKEmitter.emit('changeEditMenu', {MenuID: card.setting.menu})
      }
    } else if (card.setting.click === 'menus' && card.menus && card.menus.length > 0 && cards.subtype === 'datacard' && appType) {
      this.setState({visible: true})
    }
  }

  changeMenu = (MenuId) => {
    if (MenuId === 'IM') {
      if (!sessionStorage.getItem('instantMessage')) return

      let param = {
        MenuID: sessionStorage.getItem('instantMessage'),
        copyMenuId: '',
        type: 'view'
      }
  
      param = window.btoa(window.encodeURIComponent(JSON.stringify(param)))
  
      MKEmitter.emit('changeEditMenu', {routerUrl: '/imdesign/' + param})
    } else {
      MKEmitter.emit('changeEditMenu', {MenuID: MenuId})
    }
  }

  render() {
    const { cards } = this.props
    const { card, elements, side, visible, timestamp } = this.state

    let _style = {...card.style}

    if (side === 'back') {
      _style = {
        ...card.backStyle,
        height: card.style.height,
        marginBottom: card.style.marginBottom,
        marginLeft: card.style.marginLeft,
        marginRight: card.style.marginRight,
        marginTop: card.style.marginTop
      }
    }
    _style = resetStyle(_style)

    let checkAll = ''
    if ((cards.subtype === 'datacard' || cards.subtype === 'dualdatacard') && card.$cardType === 'extendCard') {
      checkAll = card.setting.checkAll === 'show' ? ' mk-checkable mk-extend-card' : ''
      if (checkAll && cards.wrap.selStyle === 'check square') {
        checkAll = ' mk-checkable square mk-extend-card'
      }
    } else if (cards.subtype === 'datacard') {
      if (cards.wrap.selStyle === 'check') {
        checkAll = ' mk-checkable'
      } else if (cards.wrap.selStyle === 'check square') {
        checkAll = ' mk-checkable square'
      }
    }

    let tablerole = ''
    if (card.$cardType === 'extendCard' && card.setting.cardRole === 'header') {
      tablerole = ' mk-table-header'
    }

    let _style_ = null

    if (card.style.clear === 'left') {
      _style_ = {clear: 'left'}
    } else if (card.style.clear === 'right') {
      _style_ = {float: 'right'}
    }

    return (
      <Col span={card.setting.width || 6} style={_style_}>
        <div className={'card-item ' + (card.setting.btnControl || '') + checkAll + tablerole} style={_style} onDoubleClick={(e) => {e.stopPropagation(); this.doubleClickCard()}} id={card.uuid}>
          <span className="circle-select"></span>
          <CardCellComponent cards={cards} cardCell={card} side={side} timestamp={timestamp} elements={elements} updateElement={this.updateCard}/>
          <div className="card-control" onDoubleClick={(e) => e.stopPropagation()}>
            <Popover overlayClassName="mk-popover-control-wrap" mouseLeaveDelay={0.2} mouseEnterDelay={0.2} content={
              <div className="mk-popover-control">
                <PlusOutlined className="plus" title="添加元素" onClick={this.addElement} />
                {!tablerole ? <PlusSquareOutlined className="plus" title="添加按钮" onClick={this.addButton} /> : null}
                <NormalForm title={cards.subtype === 'datacard' && card.$cardType !== 'extendCard' ? '循环卡片设置' : '属性卡片设置'} width={950} update={this.updateSetting} getForms={this.getSettingForms}>
                  <EditOutlined className="edit" title="编辑"/>
                </NormalForm>
                <CopyComponent type="cardcell" card={card}/>
                <PasteController options={['action', 'customCardElement', 'simpleform', 'forms', 'formgroup', 'cardcell']} updateConfig={this.paste} />
                <FontColorsOutlined className="style" title="调整样式" onClick={this.changeStyle} />
                <Popover overlayClassName="mk-popover-control-wrap" mouseLeaveDelay={0.2} mouseEnterDelay={0.2} content={
                  <div className="mk-popover-control">
                    <ArrowLeftOutlined className="plus" title="前移" onClick={() => this.props.move(card, 'left')} />
                    <ArrowRightOutlined className="close" title="后移" onClick={() => this.props.move(card, 'right')} />
                  </div>
                } trigger="hover" getPopupContainer={() => document.getElementById(card.uuid + 'swap')}>
                  <SwapOutlined id={card.uuid + 'swap'}/>
                </Popover>
                {cards.subtype === 'propcard' || card.$cardType === 'extendCard' ? <DeleteOutlined className="close" title="删除卡片" onClick={() => this.props.deleteElement(card)} /> : null}
                {card.setting.type === 'multi' ? <Switch size="small" onClick={this.changeSide} defaultChecked /> : null}
              </div>
            } trigger="hover">
              <ToolOutlined style={cards.subtype === 'datacard' && card.$cardType === 'extendCard' ? {color: '#26C281'} : null}/>
            </Popover>
          </div>
        </div>
        <Modal
          title="菜单组"
          wrapClassName="menus-detail-modal"
          visible={visible}
          closable={false}
          width={900}
          maskClosable={false}
          footer={[<Button key="close" onClick={() => { this.setState({ visible: false })}}>关闭</Button>]}
          destroyOnClose
        >
          <div className="menu-line">
            <div className="sort">序号</div>
            <div className="sign">标识</div>
            <div className="name">菜单</div>
            <div className="action">操作</div>
          </div>
          {card.menus && card.menus.map((item, index) => {
            return <div className="menu-line" key={index}>
              <div className="sort">{index + 1}</div>
              <div className="sign">{item.sign}</div>
              <div className="name">{item.label}</div>
              <div className="action"><span onClick={() => this.changeMenu(item.menu)}>详情</span></div>
            </div>
          })}
        </Modal>
      </Col>
    )
  }
}

export default CardBoxComponent